%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author: Roland Miezianko (rmiezian@astro.temple.edu)
% Date: October 21, 2002
% Instructor: Dr. Longin Jan Latecki
% Course: CIS 581
% Project : Assignment 2 (1.7 Template matching (Ex. #7, p. 397 in [1]). (Roland))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [FoundRow,FoundCol,FoundCorr] = findTemplate(SourceFile,TemplateFile,DataStoreFile)
% findTemplate Finds template image inside source image.
% findTemplate(SourceFile,TemplateFile,DataStoreFile) source image file, template image file,
% output correlation file for correlation map

% row and column of the template match and its corroletion value
FoundRow = -1;
FoundCol = -1;
FoundCorr = -2; % this value will be set between -1 and +1

% read the template image
% read original gray-scale template image from file
imTemp = imread(TemplateFile);

% convert template image to doubles
imTempd = double(imTemp);
[TempRows,TempCols] = size(imTempd);
TempMean = mean(mean(imTempd));

% read the source image
% read original gray-scale source image from file
imSrc = imread(SourceFile);

% convert source image to doubles
imSrcd = double(imSrc);
[SrcRows,SrcCols] = size(imSrcd);
% SrcMean = mean(mean(imSrcd));

% create correlation matrix
% its size is source - template
CorrM = zeros(SrcRows-TempRows,SrcCols-TempCols);

% find the correlation matrix
% assumption: template is entirely inside the source image

for r = 1:(SrcRows-TempRows)
    disp(r) % show the row we are working with...
    for c = 1:(SrcCols-TempCols)

% for r = 40:44 % test know location 42 for F01.bmp
%     disp(r) % show the row we are working with...
%     for c = 52:56 % test known location 54 for F01.bmp

        % reset the correlation values for this row-col location
        CorrNom = 0;
        CorrDen1 = 0;
        CorrDen2 = 0;

        % get the source average level for the subsection covered in this
        % template
        SrcMean = mean(mean(imSrcd(r:r+TempRows-1,c:c+TempCols-1)));
                
        % at this row-col location, find the correlation
        % of the template to the source image
        for rl = 0:(TempRows-1)
            for cl = 0:(TempCols-1)
                
                % compute parts of the correlation value
                % to be used in computing CorrM(r,c)
                
                TempSub = imTempd(rl+1,cl+1)-TempMean;
                SrcSub = imSrcd(r+rl,c+cl)-SrcMean;
                
                CorrNom = CorrNom + (TempSub*SrcSub);
                CorrDen1 = CorrDen1 + (TempSub*TempSub);
                CorrDen2 = CorrDen2 + (SrcSub*SrcSub);
                
            end % template columns
        end % template rows
        
        % store the correlation value in a matrix
        % for analysis
        % this value is between -1 and +1
        CorrM(r,c) = CorrNom / ( sqrt(CorrDen1*CorrDen2) );
        
        % check if correlation for this row and column is better
        % than the prevoius correlation value 
        if FoundCorr < CorrM(r,c)
            % found better
            FoundCorr = CorrM(r,c);
            FoundRow = r;
            FoundCol = c;
        end
        
    end % source columns
end % source rows

% save correlation data matrix to external file
save(DataStoreFile, 'CorrM', '-ASCII'); % to load back use:  x=load('CorrMdata.m');

return;
