%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author: Roland Miezianko (rmiezian@astro.temple.edu)
% Date: October 21, 2002
% Instructor: Dr. Longin Jan Latecki
% Course: CIS 581
% Project : Assignment 2 (1.7 Template matching (Ex. #7, p. 397 in [1]). (Roland))
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reset data and display
clear all
clc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TEMPLATE FOR ALL TEST DATA
TemplateFile = 'CircleTemplate.bmp';    % template file name
TemplateThreshold = 0.800;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% select test data for output
TestNumber = 1;

switch TestNumber
    
    case 0 % DEMO DATA 0
		SourceFile = 'F01.BMP';         
		DataStoreFile = 'F01Demo.txt';
        ImageFile = 'F01Temp.bmp';
        
    case 1 % TEST DATA 1
		SourceFile = 'F01.BMP';
		DataStoreFile = 'F01CorrMap.txt';
        ImageFile = 'F01CorrMap.bmp';
        
    case 2 % TEST DATA 2
		SourceFile = 'F05.BMP';
		DataStoreFile = 'F05CorrMap.txt';
        ImageFile = 'F05CorrMap.bmp';
        
    case 3 % TEST DATA 3
		SourceFile = 'F08.BMP';
		DataStoreFile = 'F08CorrMap.txt';
        ImageFile = 'F08CorrMap.bmp';
        
    case 4 % TEST DATA 4
		SourceFile = 'F06.BMP';
		DataStoreFile = 'F06CorrMap.txt';
        ImageFile = 'F06CorrMap.bmp';
     
    case 5 % TEST DATA 5 (no template circle)
		SourceFile = 'PAD1.BMP';
		DataStoreFile = 'PAD1CorrMap.txt';
        ImageFile = 'PAD1CorrMap.bmp';
        
    otherwise
        return;
end

% load test data corrolation map
CorrMap = load(DataStoreFile);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% display correlation map in 3D
figure;
colormap('default');
meshc(CorrMap);

% axis control and size
%set(gca,'ZLim',[-1 1]); % set the z limit to -1 to +1
set(gca,'YDir','reverse'); % show y axis direction from top to bottom, just like image
grid off;
title('Correlation Map','FontSize',12,'Color','b')

% find the index of the largest correlation value
[Mx,Col] = max(max(CorrMap,[],1)); % col
[Mx,Row] = max(max(CorrMap,[],2)); % row

% format and display text over the peak correlation
s1 = ['\downarrow'];
[s2,errmsg] = sprintf('  Maximum %0.3f at (%d,%d)', Mx, Row, Col);
s = strcat(s1,s2);
text(Col,Row,Mx+0.1,s,'FontSize',10,'Color','r');
% to store the correlation map as a bitmap, uncomment this...
% [X,Map] = frame2im(getframe);
% imwrite(X,ImageFile,'bmp');
% return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the template image
imTemp = imread(TemplateFile);
imTempd = double(imTemp);
[TempRows,TempCols] = size(imTempd);
figure;
colormap(gray(256));
image(imTempd);
axis image;
title('Template Image','FontSize',12,'Color','b');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the source image
imSrc = imread(SourceFile);
imSrcd = double(imSrc);
[SrcRows,SrcCols] = size(imSrcd);
figure;
colormap(gray(256));
image(imSrcd);
axis image;
title('Source Image','FontSize',12,'Color','b');
% hold on;
% contour(CorrMap);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% decide if we found a template or not
if TemplateThreshold < Mx
	% draw the found rectangle on the source image in red
	x = [Col; Col; Col+TempCols; Col+TempCols];
	y = [Row; Row+TempRows; Row+TempRows; Row];
	patch(x,y,[1 0 0],'FaceColor', 'none', 'EdgeColor', [1 0 0]);
	text(Col,Row,'X','HorizontalAlignment','center','Color','r');
	
	% format and display text over the found coordinates
	[s,errmsg] = sprintf('(%d,%d)', Row, Col);
	text(Col,Row-5,s,'HorizontalAlignment','left','VerticalAlignment','bottom','FontSize',10,'Color','r');
else
	% format and display text over the found coordinates
	[s,errmsg] = sprintf('Temeplate Not Found  %0.3f < %0.3f', Mx, TemplateThreshold);
	text(SrcCols/2,SrcRows/2,s,'HorizontalAlignment','center','VerticalAlignment','middle','FontSize',14,'Color','r');
end
% to capture the image and graphic overlay, uncomment this...
% [X,Map] = frame2im(getframe);
% imwrite(X,ImageFile,'bmp');

% show correlation function
% text(Col-10,Row-10,'r = \Sigma{(X_{i}-X_{avg})\times(Y_{i}-Y_{avg})}\div{\surd{\Sigma{(X_{i}-X_{avg})}^{2}\times\Sigma{(Y_{i}-Y_{avg})}^{2}}}','Color','r');
